#
# This file is part of python sample package for the BROADWAY2 package.
#
# raw_checker.py
#
# MIT License

# Copyright (c) 2013-2023 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from broadway2 import EthFrame, RawRxFrame

class RawChecker:
    ''' RawChecker to verify Ethernet-Raw-Streams
    Verifies and meassures:
    - SequenceCnt to detect USB-transmission problems
    - Timestamps increments
    - bandwidth in bytes/sec
    - total bytes received
    '''

    def __init__(self, mode=0):
        self.__mode = mode  #checker-Mode: 0 is default, only verifies RawHeader, no Data-Counters

        self.lastSeqCnt = -1
        self.lastTimestamp = -1
        self.lastHWTimestamp = -1
        self.lastRawFrameReceived = None

        self.llcFramesCnt = 0

        self.seqErrors = 0
        self.tmErrors = 0

        self.bytesReceived = 0
        self.packetsReceived = 0

        # LLC checker uses Type in Data: 1234, then frameId as 8-Byte Counter

        self.lastFrameId = -1
        self.dataErrors = 0

    def check_next_frame(self, eth):
        # verify frame data contents based on LLC-Protocoll
 
        if eth.EtherType < 1530:   # LLC protocol
            llc = int(eth.Data[0:4], 16)
            if llc == 0x1234:
                self.llcFramesCnt += 1
                frameId = int(eth.Data[4:12], 16)
                #print('EthFrame: LLC received: {0}'.format(frameId))

                if self.lastFrameId < 0:
                    diff = 1
                else:
                    diff = frameId - self.lastFrameId

                if diff != 1:
                    self.dataErrors += 1
                    #print('ERROR: {0} != {1}'.format(frameId, self.lastFrameId))

        
                self.lastFrameId = frameId

        

    def check_sequence(self, rawFrameList):
        # verif sequence count and measure timestamps
        
        cnt = len(rawFrameList)
        if cnt < 1:
            return 0

        firstSeq = True
        byteCnt = 0
        for rawFrame in rawFrameList:
            ethRaw = RawRxFrame(1, rawFrame)
            seqCnt = ethRaw.SequenceCnt
            byteCnt += len(rawFrame) - 16
            if firstSeq:
                if self.lastSeqCnt < 0:
                    diff = 1
                else:
                    diff = seqCnt - self.lastSeqCnt
                    if diff != 1:
                        #print('SeqErr in firstSeq: {0} / {1}'.format(diff, seqCnt))
                        self.seqErrors += 1
                firstSeq = False
            else:
                diff = seqCnt - lastSeqCnt
                if diff != 1:
                    #print('SeqErr in: {0} / {1}'.format(diff, seqCnt))
                    self.seqErrors += 1

            lastSeqCnt = seqCnt
            if self.__mode > 0:
                self.check_next_frame(ethRaw.EthFrame)
        

        self.lastSeqCnt = lastSeqCnt

        self.lastRawFrameReceived = ethRaw
        self.lastSeqCnt = seqCnt
        self.lastHWTimestamp = ethRaw.TimestampSec * 1e9 + ethRaw.TimestampNano

        self.packetsReceived += cnt
        self.bytesReceived += byteCnt

        return cnt
