#
# This file is part of python sample package for the BROADWAY2 package.
#
# raw_loop_chk.py
#
# MIT License

# Copyright (c) 2013-2023 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.



# Raw-Sender/Receiver requires echo from Link-Partner to work as expected
# Loopbacks are typically used for BitError/CRC transmit-receive verification
# Ethernet-Phy NXP TJA1101 has 3 loopback option: internal, external, remote
# Internal, External are good to start without Link-Partner - verify software flow
# External: Connect Link-Partner (e.g. another FC602/FC612 and set it to remote loopback

import sys, os
import time
import binascii
import platform
import time, datetime

from broadway2 import Broadway, broadway_api, RawReceiver, RawRxHeaderTypes, EthFrame, RawRxFrame
from raw_checker import RawChecker

DefaultInst = 0 # Use first found USB-Adapter
#DefaultPayload = 64-32     # Dummy data + LLC-Header + counter + timestamps, Payload = 32 will be 64 Bytes LLC-Data 
DefaultPayload = 1518
PayloadHeader = 32                 # First 32 Bytes are filled with TS, counters, ...

BurstCnt = 3                # Number of Frames send to bufffer to speed-up traffic

NumTxFrames = 100000 # 2.7e6


def verify_receive_simple(srcData, recvData):
    cntSrc = int(binascii.b2a_hex(srcData[16:20]), base=16)
    cntRecv = int(recvData[4:12], base=16)
    #print(cntSrc, cntRecv)
    if cntRecv == cntSrc:
        return True
    
    return False

def verify_receive_full(srcData, recvData, payloadLength):
    # srcData as hex-string. start at MAC-Tgt
    # recvData as 
    cntSrc = int(srcData[32:40], base=16)
    cntRecv = int(recvData[32:40], base=16)
    #print(cntSrc, cntRecv)
    if cntRecv == cntSrc:
        if srcData[66:payloadLength] == recvData[66:payloadLength]:
            return True
    
    return False

def main(args):

    inst = DefaultInst
    payload = DefaultPayload

    
    # create adapter singleton
    base = Broadway()

    
    aList = base.get_raw_adapters()

    if len(aList) < 1:
        print('No Raw-Adapters found!')
        sys.exit(-1)


    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 0

    inst, friendlyName = aList[sel-1]

    adapter = base.get_adapter(inst)

    macSrc = adapter.MACAddress

    devControl = base.open_control(inst)
    adapter.connect(devControl)

    # Option to delay max. speed in case of rx-overrun
    # Set minTxInterval to 1 to reduce max. Tx-Speed

    minTxInterval = 1
    res = adapter.set_min_tx_frame_interval(minTxInterval)

    print('Open Adapter to send raw: {0} / {1}'.format(friendlyName, inst))    
    print('='*30)

    # config transmit-side
    devTx = base.open_raw_tx(inst)
    print('Raw Sender opened: {0}'.format(friendlyName))

    # initialize receiver
    devRx = base.open_raw_rx(inst, RawRxHeaderTypes.HeaderType2)
    print('Raw Rx Device opened: {0} / {1}'.format(friendlyName, inst))

    rawChecker = RawChecker(1)

    rawReceiver = RawReceiver(devRx)

    rawReceiver.enable()
        
    if devTx == None:
        sys.exit(-1)

    payloadData = ''
    for j in range(0, payload-PayloadHeader):
        payloadData += '{0:02X}'.format(j & 0xFF)

    frameId = 0  # only first seq=0 has no timestamp, all following will use hw timestamp from previous send result
    hwTimeStamp = (0,0)
    print('Sending data {0}.... Press <Ctrl-C> to stop/exit'.format(payload))

    # close control while send/receive, enables to open other control tools in parallel during send/receive
    base.close(devControl)

    txCnt = 0
    rxCnt = 0
    errCnt = 0
    tmStart = datetime.datetime.now()

    try:
        while 1:
            # prepare echo frame, build out of received data
            if (rxCnt % 1000) == 0:
                print('Tx: {0} Rx: {1}   ErrCnt: {2}'.format(txCnt, rxCnt, errCnt))       

            bcnt = BurstCnt
            burstList = []
            while bcnt > 0:
                # no VLAN, use LLC default protocol for correct decoding in wireshark
                sendData = 'FFFFFFFFFFFF' + macSrc + '{0:04X}'.format(payload+18) + '1234' + '{0:08X}'.format(txCnt) + '{0:016X}'.format(hwTimeStamp[0]) + '{0:08X}'.format(hwTimeStamp[1]) + payloadData
                
                binData = binascii.a2b_hex(sendData)    # convert ascii data to binary stream

                broadway_api.submit_raw_frame(devTx, binData)   # submit to buffer
                burstList.append(binData)
                
                txCnt += 1
                bcnt -= 1

            res = broadway_api.send_raw_frames(devTx)       # send-out

            #time.sleep(0.3)

            bcnt = BurstCnt

            while bcnt > 0:

                # read_block with max. 500 frames and timeout after 500 ms
                result = rawReceiver.read_block(20, 100)
                # returns array of tuples (timestamp, seqCounter, Ethernet-Frame)
            
                for res in result:
                    ethRaw = RawRxFrame(1, res)
                    # print(' -> {0:8} {1}.{2}'.format(ethRaw.SequenceCnt, ethRaw.TimestampSec, ethRaw.TimestampNano))
                    eth = ethRaw.EthFrame
                    # print('  {0} {1}'.format(str(eth), eth.Data[0:16]))
                    binData = burstList.pop(0)
                    #res = verify_receive_simple(binascii.b2a_hex(binData), eth.Data)
                    res = verify_receive_full(binascii.b2a_hex(binData), binascii.b2a_hex(ethRaw.EthFullFrame), payload)
                    if res == False:
                        errCnt += 1
                
                    bcnt -= 1
                    rxCnt += 1

            if bcnt != 0:
                print('BBBB')
                
            if txCnt >= NumTxFrames:
                break


                
                    
    except KeyboardInterrupt:
        base.close(devTx)

    tmStop = datetime.datetime.now()

    if txCnt < NumTxFrames:
        print('='*64)
        print('!!!!! TEST - ABORTED. Not a final test to report')
        print('='*64)

    print('TX-RX-Loop finished: Duration: {0} Bytes transfered: {1} Rate: {2} kBytes/sec'.format((tmStop-tmStart), txCnt*payload, ((txCnt*payload)/(tmStop-tmStart).seconds)/1e3))
    print('Create Loopback-Report: TX:{0} RX:{1} ErrCnt: {2} Payload: {3}'.format(txCnt, rxCnt, errCnt, payload))
    print('EOT')

            

if __name__ == "__main__":
    main(sys.argv)


